//
//  RandLib.hpp
//  SynthEco
//
//

#ifndef RandLib_h
#define RandLib_h

#include <random>


//std::random_device rd;
//std::mt19937 mersenne(rd());

class RandLib
{
private:
    // Private Constructor
    RandLib() {
        std::random_device rd;
        std::array<int, std::mt19937::state_size> seed_data;
        std::generate_n(seed_data.data(), seed_data.size(), std::ref(rd));
        std::seed_seq seq(std::begin(seed_data), std::end(seed_data));
        this->mersenne = std::mt19937(seq);
    }
    // Stop the compiler generating methods of copy the object
    RandLib(RandLib const& copy);            // Not Implemented
    RandLib& operator=(RandLib const& copy); // Not Implemented
public:
    std::mt19937 mersenne;
    static RandLib& inst()
    {
        static RandLib instance;
        return instance;
    }
    vector<double> uvec(unsigned int length, double low=0, double high=1) {
        vector<double> q(length);
        std::uniform_real_distribution<double> distrib(0,1);
        std::transform(q.begin(), q.end(), q.begin(),  [&distrib](double v) {
            return distrib(RandLib::inst().mersenne);
        });
        return q;
    }
    template<class T=double>
    T randn(T mean, T stddev) {
        return std::normal_distribution<T>(mean, stddev)(RandLib::inst().mersenne);
    }
    template<class T=double>
    T randu(T low, T high) {
        return std::uniform_real_distribution<T>(low, high)(RandLib::inst().mersenne);
    }
    
    vector<int> randRange(int start, int end) {
        std::vector<int> arr(end-start);
        std::iota(arr.begin(), arr.end(), start);
        std::shuffle(arr.begin(), arr.end(), RandLib::inst().mersenne);
        return arr;
    }
    
};
#endif /* RandLib_h */
